//!
//! # Delete object
//!
//!

use std::fmt::Debug;

use fluvio_protocol::{Encoder, Decoder};
use fluvio_protocol::api::Request;

use crate::{DeletableAdminSpec};
use crate::Status;
use crate::AdminPublicApiKey;
use super::{DeleteApiEnum, COMMON_VERSION};

DeleteApiEnum!(DeleteRequest);

// This can be auto generated by enum derive later
#[derive(Debug, Default, Encoder, Decoder)]
pub struct DeleteRequest<S: DeletableAdminSpec> {
    key: S::DeleteKey,
}

impl<S> DeleteRequest<S>
where
    S: DeletableAdminSpec,
{
    pub fn new(key: S::DeleteKey) -> Self {
        Self { key }
    }

    pub fn key(self) -> S::DeleteKey {
        self.key
    }
}

impl Request for ObjectApiDeleteRequest {
    const API_KEY: u16 = AdminPublicApiKey::Delete as u16;
    const MIN_API_VERSION: i16 = 1; // previous version
    const DEFAULT_API_VERSION: i16 = COMMON_VERSION;
    type Response = Status;
}
